@@ -5,9 +5,15 @@
 /* vim: set ts=4 sw=4 wm=5 : */
 
 #include <gtk/gtk.h>
+#include <libintl.h>
+#include <locale.h>
 #include "../config.h"
 #include "gtkspell.h"
 
+#define _(String) dgettext (PACKAGE, String)
+
+#define GTKSPELL_MISSPELLED_TAG "gtkspell-misspelled"
+
 #ifdef HAVE_ASPELL_H
    #define USING_ASPELL
    #include <aspell.h>
@@ -374,7 +380,7 @@ build_suggestion_menu(GtkSpell *spell, GtkTextBuffer *buffer,
 		/* no suggestions.  put something in the menu anyway... */
 		GtkWidget *label;
 		label = gtk_label_new("");
-		gtk_label_set_markup(GTK_LABEL(label), "<i>(no suggestions)</i>");
+		gtk_label_set_markup(GTK_LABEL(label), _("<i>(no suggestions)</i>"));
 
 		mi = gtk_menu_item_new();
 		gtk_container_add(GTK_CONTAINER(mi), label);
@@ -388,7 +394,7 @@ build_suggestion_menu(GtkSpell *spell, GtkTextBuffer *buffer,
 				gtk_widget_show(mi);
 				gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
 
-				mi = gtk_menu_item_new_with_label("More...");
+				mi = gtk_menu_item_new_with_label(_("More..."));
 				gtk_widget_show(mi);
 				gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
 
@@ -414,7 +420,7 @@ build_suggestion_menu(GtkSpell *spell, GtkTextBuffer *buffer,
 	gtk_menu_shell_append(GTK_MENU_SHELL(topmenu), mi);
 
 	/* + Add to Dictionary */
-	label = g_strdup_printf("Add \"%s\" to Dictionary", word);
+	label = g_strdup_printf(_("Add \"%s\" to Dictionary"), word);
 	mi = gtk_image_menu_item_new_with_label(label);
 	g_free(label);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), 
@@ -425,7 +431,7 @@ build_suggestion_menu(GtkSpell *spell, GtkTextBuffer *buffer,
 	gtk_menu_shell_append(GTK_MENU_SHELL(topmenu), mi);
 
 	/* - Ignore All */
-	mi = gtk_image_menu_item_new_with_label("Ignore All");
+	mi = gtk_image_menu_item_new_with_label(_("Ignore All"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), 
 			gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU));
 	g_signal_connect(G_OBJECT(mi), "activate",
@@ -458,7 +464,7 @@ populate_popup(GtkTextView *textview, GtkMenu *menu, GtkSpell *spell) {
 
 	/* then, on top of it, the suggestions menu. */
 	img = gtk_image_new_from_stock(GTK_STOCK_SPELL_CHECK, GTK_ICON_SIZE_MENU);
-	mi = gtk_image_menu_item_new_with_label("Spelling Suggestions");
+	mi = gtk_image_menu_item_new_with_label(_("Spelling Suggestions"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
 
 	word = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
@@ -495,6 +501,20 @@ button_press_event(GtkTextView *view, GdkEventButton *event, GtkSpell *spell) {
 					 we don't want to eat any events. */
 }
 
+/* This event occurs when the popup menu is requested through a key-binding
+ * (Menu Key or <shift>+F10 by default).  In this case we want to set
+ * spell->mark_click to the cursor position. */
+static gboolean
+popup_menu_event(GtkTextView *view, GtkSpell *spell) {
+	GtkTextIter iter;
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer(view);
+
+	gtk_text_buffer_get_iter_at_mark(buffer, &iter, 
+			gtk_text_buffer_get_insert(buffer));
+	gtk_text_buffer_move_mark(buffer, spell->mark_click, &iter);
+	return FALSE; /* false: let gtk process this event, too. */
+}
+
 static gboolean
 gtkspell_set_language_internal(GtkSpell *spell, const gchar *lang, GError **error) {
 	AspellConfig *config;
@@ -593,10 +613,15 @@ gtkspell_recheck_all(GtkSpell *spell) {
 GtkSpell*
 gtkspell_new_attach(GtkTextView *view, const gchar *lang, GError **error) {
 	GtkTextBuffer *buffer;
+	GtkTextTagTable *tagtable;
 	GtkTextIter start, end;
 
 	GtkSpell *spell;
 
+#ifdef ENABLE_NLS
+	bindtextdomain(PACKAGE, LOCALEDIR);
+#endif
+
 	if (error)
 		g_return_val_if_fail(*error == NULL, NULL);
 
@@ -618,6 +643,8 @@ gtkspell_new_attach(GtkTextView *view, const gchar *lang, GError **error) {
 			G_CALLBACK(button_press_event), spell);
 	g_signal_connect(G_OBJECT(view), "populate-popup",
 			G_CALLBACK(populate_popup), spell);
+	g_signal_connect(G_OBJECT(view), "popup-menu",
+			G_CALLBACK(popup_menu_event), spell);
 
 	buffer = gtk_text_view_get_buffer(view);
 
@@ -634,11 +661,20 @@ gtkspell_new_attach(GtkTextView *view, const gchar *lang, GError **error) {
 			"mark-set",
 			G_CALLBACK(mark_set), spell);
 
-	spell->tag_highlight = gtk_text_buffer_create_tag(buffer,
-			"gtkspell-misspelled",
-			"foreground", "red", 
-			"underline", PANGO_UNDERLINE_SINGLE,
-			NULL);
+	tagtable = gtk_text_buffer_get_tag_table(buffer);
+	spell->tag_highlight = gtk_text_tag_table_lookup(tagtable, GTKSPELL_MISSPELLED_TAG);
+
+	if (spell->tag_highlight == NULL) {
+		spell->tag_highlight = gtk_text_buffer_create_tag(buffer,
+				GTKSPELL_MISSPELLED_TAG,
+#ifdef HAVE_PANGO_UNDERLINE_ERROR
+				"underline", PANGO_UNDERLINE_ERROR,
+#else
+				"foreground", "red", 
+				"underline", PANGO_UNDERLINE_SINGLE,
+#endif
+				NULL);
+	}
 
 	/* we create the mark here, but we don't use it until text is
 	 * inserted, so we don't really care where iter points.  */
